<?php
/*
 * Plugin Name: GOAT One Payment Gateway for WooCommerce
 * Plugin URI: https://goatone.com/goatone-woocommerce
 * Description: Accept credit card payments on your WooCommmerce store using GOAT One Payment Gateway.
 * Author: GOAT One
 * Author URI: https://goatone.com/
 * Version: 1.0
 * Requires at least: 4.0
 * Tested up to: 4.9
 * WC requires at least: 2.5
 * WC tested up to: 3.2.6
 *
 * Copyright (c) 2018 GOAT One
 *
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


if (!class_exists('WC_GOATOne')) {

    class WC_GOATOne {

        var $version = '1.0';
        var $php_version = '2.5.0';
        var $wc_version = '2.5.0';
        var $plugin_url;
        var $plugin_path;
        var $notices = array();

        function __construct() {
            $this->define_constants();
            $this->includes();
            $this->loader_operations();
            //Handle any db install and upgrade task
            add_action( 'admin_init', array( $this, 'check_environment' ) );
            add_action( 'admin_notices', array( $this, 'admin_notices' ), 15 );
            add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'plugin_action_links' ) );
            add_action('admin_enqueue_scripts', array( $this,'goat_register_assets'));
        }
        function goat_register_assets(){
        	wp_enqueue_style( 'goat_admin_css', plugins_url('assets/css/wc-goat.css', __FILE__) );
        	wp_enqueue_script( 'goat_admin_script', plugin_dir_url( __FILE__ ) . 'assets/js/goat-request.js', array('jquery'), '1.0' );
        }

        function define_constants() {
            define('WC_GOATOne_VERSION', $this->version);
            define('WC_GOATOne_MIN_PHP_VER', $this->php_version );
            define('WC_GOATOne_MIN_WC_VER', $this->wc_version );
            define('WC_GOATOne_PLUGIN_URL', $this->plugin_url());
            define('WC_GOATOne_PLUGIN_PATH', $this->plugin_path());
        }

        function includes() {
        	include_once( dirname( __FILE__ ) . '/includes/class-wc-utility-goatone.php' );
        	include_once( dirname( __FILE__ ) . '/license/license-management.php' );
        }

        function loader_operations() {
            add_action('plugins_loaded', array(&$this, 'plugins_loaded_handler')); //plugins loaded hook		
        }

        function plugins_loaded_handler() {
            //Runs when plugins_loaded action gets fired
            include_once( dirname( __FILE__ ) . '/includes/class/BankCardTransaction.php' );
			include_once( dirname( __FILE__ ) . '/includes/class/BaseCommerceClient.php' );
			include_once( dirname( __FILE__ ) . '/includes/class-wc-gateway-goatone.php' );
            add_filter('woocommerce_payment_gateways', array(&$this, 'init_goatone_gateway'));
        }
        

        function check_environment() {
        	echo '<script>var apurl = "'.admin_url("admin-post.php").'";</script>';
            if (  WC_GOATOne_VERSION !== get_option( 'wc_goatone_version' ) )  {
				$this->_update_plugin_version();
			}
			$environment_warning = $this->get_environment_warning();

			if ( $environment_warning && is_plugin_active( plugin_basename( __FILE__ ) ) ) {
				$this->add_admin_notice( 'bad_environment', 'error', $environment_warning );
			}
			$options = get_option( 'woocommerce_goatone_settings' );
			//$secret = $options['sdk_key'];
            if ((isset($options['sdk_key']) && empty( $options['sdk_key'])) && ! ( isset( $_GET['page'], $_GET['section'] ) && 'wc-settings' === $_GET['page'] && 'goatone' === $_GET['section'] ) ) {
				$setting_link = $this->get_setting_link();
				$this->add_admin_notice( 'prompt_connect', 'notice notice-warning', sprintf( __( 'GOAT One Payment Gateway is almost ready. To get started, <a href="%s">enter your SDK Credentials. </a>.', 'woocommerce-gateway-goatone' ), $setting_link ) );
			}
        }

        private static function _update_plugin_version() {
			delete_option( 'wc_goatone_version' );
			update_option( 'wc_goatone_version', WC_GOATOne_VERSION );
			return true;
		}

        function plugin_url() {
            if ($this->plugin_url)
                return $this->plugin_url;
            return $this->plugin_url = plugins_url(basename(plugin_dir_path(__FILE__)), basename(__FILE__));
        }

        function plugin_path() {
            if ($this->plugin_path)
                return $this->plugin_path;
            return $this->plugin_path = untrailingslashit(plugin_dir_path(__FILE__));
        }



        function plugin_action_links($links){
           $setting_link = $this->get_setting_link();

			$plugin_links = array(
				'<a href="' . $setting_link . '">' . __( 'Settings', 'woocommerce-gateway-goatone' ) . '</a>',
				'<a href="https://goatone.com/goatone-woocommerce/">' . __( 'Docs', 'woocommerce-gateway-goatone' ) . '</a>',
				'<a href="https://goatone.com/goatone-woocommerce-support/">' . __( 'Support', 'woocommerce-gateway-goatone' ) . '</a>',
			);
			return array_merge( $plugin_links, $links );        
        }

        function get_setting_link() {
			$use_id_as_section = function_exists( 'WC' ) ? version_compare( WC()->version, '2.6', '>=' ) : false;

			$section_slug = $use_id_as_section ? 'goatone' : strtolower( 'WC_Gateway_GOATOne' );

			return admin_url( 'admin.php?page=wc-settings&tab=checkout&section=' . $section_slug );
		}
        
        function init_goatone_gateway($methods) {        
			$methods[] = 'WC_Gateway_GOATOne';			
			return $methods;
        }
        /**
		 * Allow this class and other classes to add slug keyed notices (to avoid duplication)
		 */
		function add_admin_notice( $slug, $class, $message ) {
			$this->notices[ $slug ] = array(
				'class'   => $class,
				'message' => $message,
			);
		}
		/**
		 * Display any notices we've collected thus far (e.g. for connection, disconnection)
		 */
	    function admin_notices() {
			foreach ( (array) $this->notices as $notice_key => $notice ) {
				echo "<div class='" . esc_attr( $notice['class'] ) . "'><p>";
				echo wp_kses( $notice['message'], array( 'a' => array( 'href' => array() ) ) );
				echo '</p></div>';
			}
		}
        /**
		 * Checks the environment for compatibility problems.  Returns a string with the first incompatibility
		 * found or false if the environment has no problems.
		 */
		function get_environment_warning() {
			if ( version_compare( phpversion(), WC_GOATOne_MIN_PHP_VER, '<' ) ) {
				$message = __( 'GOAT One Payment Gateway - The minimum PHP version required for this plugin is %1$s. You are running %2$s.', 'woocommerce-gateway-goatone' );

				return sprintf( $message, WC_GOATOne_MIN_PHP_VER, phpversion() );
			}

			if ( ! defined( 'WC_VERSION' ) ) {
				return __( 'GOAT One Payment Gateway requires WooCommerce to be activated to work.', 'woocommerce-gateway-goatone' );
			}

			if ( version_compare( WC_VERSION, WC_GOATOne_MIN_WC_VER, '<' ) ) {
				$message = __( 'GOAT One Payment Gateway - The minimum WooCommerce version required for this plugin is %1$s. You are running %2$s.', 'woocommerce-gateway-goatone' );

				return sprintf( $message, WC_GOATOne_MIN_WC_VER, WC_VERSION );
			}

			if ( ! function_exists( 'curl_init' ) ) {
				return __( 'GOAT One Payment Gateway - cURL is not installed.', 'woocommerce-gateway-goatone' );
			}

			return false;
		}
		

    }

    //End of plugin class
}//End of class not exists check

$GLOBALS['WC_GOATOne'] = new WC_GOATOne();