<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if (!class_exists('WC_Payment_Gateway')){
    //Woocommerce is not active.
    return;
}

class WC_Gateway_GOATOne extends WC_Payment_Gateway_CC {

    protected $order = null;
    protected $transactionId = null;
    protected $transactionErrorMessage = null;
    protected $sdk_username = '';
    protected $sdk_password = '';
    protected $sdk_key = '';
    protected $testmode;
    public $goatone_checkout;

    public function __construct() {
        $this->id                   = 'goatone';
		$this->method_title         = __( 'GOAT One', 'goatone-woocommerce-payment-gateway' );
		$status_val   = check_license_status();
		$this->method_description = '';
		if(isset($_GET['error']) && $_GET['error']=='yes'){
			$this->method_description .= '<div class="notice notice-warning"><p>Something went wrong! Please check your license key and try again.</p></div> ';
		}
		if(isset($_GET['deactive']) && $_GET['deactive']=='yes'){
			$this->method_description .= '<div class="notice notice-success is-dismissible"><p>Your License was deactivated successfully.</p></div> ';
		}
		if ( $status_val != 'active'){
			$this->method_description .= '<div class="license-warning notdone"><h2>Almost finished!</h2><p>Your GOAT One Payment Gateway Plugin is <span class="red">not licensed</span>.</p><p>Click the Verify button to enable the ability to accept real credit card transactions.</p><p><a href="https://goatone.com/goatone-woocommerce" target="_blank">Learn more</a> about getting a test account along with help and tips on how to finish setting up the GOAT One Payment Plugin.</p></div> ';
		 } else { 	       
		  	$this->method_description .= '<div class="license-warning done"><h2>Congratulations, your plugin is licensed!</h2><p><a href="https://goatone.com/goatone-woocommerce" target="_blank">Learn more</a> about getting a test account along with help and tips on how to finish setting up the GOAT One Payment Plugin.</p><p><a class="button" href="'.wp_nonce_url( admin_url( 'admin-ajax.php?action=goat_license_deactivate'), 'goat-deactivate-nonce' ).'">Deactivate</a></p></div> ';
		}
		$this->has_fields           = true;
		$this->view_transaction_url = 'https://my.basecommercesandbox.com/%s';
		
		// Load the form fields.
		$this->init_form_fields();

		// Load the settings.
		$this->init_settings();

		// Get setting values.
		$this->title                   = $this->get_option( 'title' );
		$this->description             = $this->get_option( 'description' );
		$this->enabled                 = $this->get_option( 'enabled' );
		$this->testmode                = 'no' === $this->get_option( 'livemode' );
		$this->sdk_username            = $this->get_option( 'sdk_username' );
		$this->sdk_password            = $this->get_option( 'sdk_password' );
		$this->sdk_key                 = $this->get_option( 'sdk_key' );
		$this->logging                 = 'yes' === $this->get_option( 'logging' );

		
		if ( $this->testmode ) {
			$this->description .= ' ' . sprintf( __( '<p>SANDBOX MODE ENABLED.</p><p>You can use the card number 4111 1111 1111 1111 with any CVC and a valid expiration date.</p>', 'goatone-woocommerce-payment-gateway' ));
			$this->description  = trim( $this->description );
		}

		add_action( 'admin_notices', array( $this, 'admin_notices' ) );
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );		
        
    }
  

    /**
	 * Check if SSL is enabled and notify the user
	 */
    public function admin_notices() {
		if ($this->sdk_key && get_option('woocommerce_force_ssl_checkout') == 'no' && $this->enabled == 'yes'){
            echo '<div class="error"><p>' . sprintf(__('%s gateway requires SSL certificate for better security. The <a href="%s">force SSL option</a> is disabled on your site. Please ensure your server has a valid SSL certificate so you can enable the SSL option on your checkout page.', 'woocommerce'), $this->method_title, admin_url('admin.php?page=wc-settings&tab=checkout')) . '</p></div>';
        }
	}

	/**
	 * Check if this gateway is enabled
	 */
	public function is_available() {
		if ( 'yes' === $this->enabled ) {
			if ( ! $this->sdk_username || ! $this->sdk_password || ! $this->sdk_key ) {
				return false;
			}
			return true;
		}
		return false;
	}
    
    public function init_form_fields() {
        $this->form_fields = include( 'settings-goatone.php' );
    }

    /*
     * Validates the fields specified in the payment_fields() function.
     */
    public function validate_fields() {
        global $woocommerce;        
   
        if (!WC_GOATONE_Utility::is_valid_card_number($_POST['goatone-card-number'])){
            wc_add_notice(__('<strong>Credit Card Number</strong> is invalid.', 'woocommerce'), 'error');
        }        
        if (!WC_GOATONE_Utility::is_valid_expiry($_POST['goatone-card-expiry'])){
            wc_add_notice(__('<strong>Card Expiration Date</strong> is not valid.', 'woocommerce'), 'error');
        }
        if (!WC_GOATONE_Utility::is_valid_cvv_number($_POST['goatone-card-cvc'])){
            wc_add_notice(__('<strong>Card Verification Number (CVV)</strong> is not valid. You can find this number on your credit card.', 'woocommerce'), 'error');
        }
    }
    
    /*
     * Render the credit card fields on the checkout page
     */
	public function payment_fields() {

		echo '<div id="goatone-payment-data">';

		if ( $this->description ) {
			echo apply_filters( 'wc_goatone_description', wpautop( wp_kses_post( $this->description ) ) );
		}

		if ( ! $this->goatone_checkout ) {
			$this->form();
		}

		echo '</div>';
	}

    public function process_payment($order_id) {
        global $woocommerce;
        $this->order = new WC_Order($order_id);
        if (  $this->order->get_total() > 0 ) {
	        $gatewayRequestData = $this->create_basecommerce_request();

	        if ($gatewayRequestData AND $this->verify_basecommerce_payment($gatewayRequestData)) {
	            $this->do_order_complete_tasks();

	            return array(
	                'result' => 'success',
	                'redirect' => $this->get_return_url($this->order)
	            );
	        } else {
	            $this->mark_as_failed_payment();
	            wc_add_notice(__('(Transaction Error) something is wrong.', 'woocommerce'),'error');
	        }
    	} else {
    		$this->order->payment_complete();
    	}
    }

    protected function mark_as_failed_payment() {
        $this->order->add_order_note(sprintf("Credit Card Payment Failed with message: '%s'", $this->transactionErrorMessage));
    }

    protected function do_order_complete_tasks() {
        global $woocommerce;

        if ($this->order->status == 'completed')
            return;

        $this->order->payment_complete();
        $woocommerce->cart->empty_cart();

        $this->order->add_order_note(
                sprintf("Credit Card payment completed with Transaction Id of '%s'", $this->transactionId)
        );

        unset($_SESSION['order_awaiting_payment']);
    }

    protected function verify_basecommerce_payment($gatewayRequestData) {
        global $woocommerce;

        $username = $gatewayRequestData['USER'];
		$s_password = $gatewayRequestData['PWD'];
		$s_key = $gatewayRequestData['KEY'];

		$cardNumber = $gatewayRequestData['ACCT'];
		$cardMonth = $gatewayRequestData['EXPMONTH'];
		$cardYear = $gatewayRequestData['EXPYEAR'];
		$cardCvv = $gatewayRequestData['CVV'];
		$amount = $gatewayRequestData['AMT'];

		$cardname = $gatewayRequestData['FIRSTNAME'].' '.$gatewayRequestData['LASTNAME'].'-'.$gatewayRequestData['OID'];

		$o_bct = new BankCardTransaction();
		$o_bct->setType(BankCardTransaction::$XS_BCT_TYPE_SALE);
		$o_bct->setCardName($cardname);
		$o_bct->setCardNumber($cardNumber);
		$o_bct->setCardExpirationMonth($cardMonth);
		$o_bct->setCardExpirationYear($cardYear);
		$o_bct->setCardCVV2($cardCvv);
		$o_bct->setAmount($amount);

		$o_bcpc = new BaseCommerceClient( $username, $s_password, $s_key );
		if ( $this->testmode ) {
			$o_bcpc->setSandbox( true );
		}
		$o_bct = $o_bcpc->processBankCardTransaction( $o_bct );

		if( $o_bct->isStatus( BankCardTransaction::$XS_BCT_STATUS_FAILED ) ) {
		    //Transaction Failed, look at messages array for reason(s) why
		   $errArray = $o_bct->getMessages();
		   if($errArray){
		   	if(is_array($errArray)){
		   		foreach ($errArray as $err) {
				 wc_add_notice($err,'error');
				}			
			   	$this->transactionErrorMessage =  implode (", ", $errArray);
		   	} else {
		   		wc_add_notice($errArray,'error');
		   		$this->transactionErrorMessage =  $errArray;
		   	}		   	
		   	return false;
		   }
		} else if( $o_bct->isStatus( BankCardTransaction::$XS_BCT_STATUS_DECLINED ) ) {
		    //Transaction Declined, look at response code and response message		    
		    $errArray = $o_bct->getResponseMessage(); 
		    if($errArray){
		   	if(is_array($errArray)){
		   		foreach ($errArray as $err) {
				 wc_add_notice($err,'error');
				}			
			   	$this->transactionErrorMessage =  implode (", ", $errArray);
		   	} else {
		   		wc_add_notice($errArray,'error');
		   		$this->transactionErrorMessage =  $errArray;
		   	}		   	
		   	return false;
		   }
		} else if( $o_bct->isStatus(BankCardTransaction::$XS_BCT_STATUS_CAPTURED) ) {
		    //Transaction went through successfully
		    $this->transactionId = $o_bct->getTransactionID(); 
		    return true;
		}

		return false;

    }


    protected function create_basecommerce_request() {
        if ($this->order AND $this->order != null) {
        	$exp_date         = explode( "/", sanitize_text_field($_POST['goatone-card-expiry']));
			$exp_month        = str_replace( ' ', '', $exp_date[0]);
			$exp_year         = str_replace( ' ', '',$exp_date[1]);
			if (strlen($exp_year) == 2) {
				$exp_year += 2000;
			}
            return array(
            	'OID' => $this->order->id,
                'PWD' => $this->sdk_password,
                'USER' => $this->sdk_username,
                'KEY' => $this->sdk_key,
                'AMT' => $this->order->get_total(),
                'FIRSTNAME' => $this->order->billing_first_name,
                'LASTNAME' => $this->order->billing_last_name,
                'IPADDRESS' => $_SERVER['REMOTE_ADDR'],
                'ACCT' => $_POST['goatone-card-number'],
                'CVV' => $_POST['goatone-card-cvc'],
                'EXPMONTH' => $exp_month,
                'EXPYEAR' => $exp_year,
                'CURRENCYCODE' => get_woocommerce_currency()
            );
        }
        return false;
    }
    
}//End of class