<?php

/**
 * Entity housing attributes of IVR customer payment
 */
class CustomerPayment {
    
    private $is_customer_id;
    private $is_source_did;
    private $is_destination_did;
    private $is_customer_account_number;
    private $is_customer_zip_code;
    
    private $id_payment_amount;
    private $in_bank_account_transaction_id;
    private $in_bank_card_transaction_id;
    
    private $is_session_id;
    
    /**
     * Default constructor
     */
    function __construct() {
        
    }
    
    /**
     * Return the value of customer ID
     * 
     * @return the value of customer ID
     */
    function getCustomerID() {
        return $this->is_customer_id;
    }
    
    /**
     * Sets the value of customer ID
     * 
     * @param $vs_customer_id new value of customer ID
     */
    function setCustomerID( $vs_customer_id ) {
        $this->is_customer_id = $vs_customer_id;
    }
    
    /**
     * Return the value of source DID
     * 
     * @return the value of source DID
     */
    function getSourceDID() {
        return $this->is_source_did;
    }
    
    /**
     * Sets the value of source DID
     * 
     * @param $vs_source_did new value of source DID
     */
    function setSourceDID( $vs_source_did ) {
        $this->is_source_did = $vs_source_did;
    }
    
    /**
     * Return the value of destination DID
     * 
     * @return the value of destination DID
     */
    function getDestinationDID() {
        return $this->is_destination_did;
    }
    
    /**
     * Sets the value of destination DID
     * 
     * @param $vs_destination_did new value of destination DID
     */
    function setDestinationDID( $vs_destination_did ) {
        $this->is_destination_did = $vs_destination_did;
    }
    
    /**
     * Return the value of customer account number
     * 
     * @return the value of customer account number
     */
    function getCustomerAccountNumber() {
        return $this->is_customer_account_number;
    }
    
    /**
     * Sets the value of customer account number
     * 
     * @param $vs_customer_account_number new value of customer account number
     */
    function setCustomerAccountNumber( $vs_customer_account_number ) {
        $this->is_customer_account_number = $vs_customer_account_number;
    }
    
    /**
     * Return the value of customer zip code
     * 
     * @return the value of customer zip code
     */
    function getCustomerZipCode() {
        return $this->is_customer_zip_code;
    }
    
    /**
     * Sets the value of customer zip code
     * 
     * @param $vs_customer_zip_code new value of customer zip code
     */
    function setCustomerZipCode( $vs_customer_zip_code ) {
        $this->is_customer_zip_code = $vs_customer_zip_code;
    } 
    
    /**
     * Return the value of payment amount
     * 
     * @return the value of payment amount
     */
    function getPaymentAmount() {
        return $this->id_payment_amount;
    }
    
    /**
     * Sets the value of payment amount
     * 
     * @param $vd_payment_amount new value of payment amount
     */
    function setPaymentAmount( $vd_payment_amount ) {
        $this->id_payment_amount = $vd_payment_amount;
    }
    
    /**
     * Return the value of bank card transaction ID
     * 
     * @return the value of bank card transaction ID
     */
    function getBankCardTransactionID() {
        return $this->in_bank_card_transaction_id;
    }
    
    /**
     * Sets the value of bank card transaction ID
     * 
     * @param $vn_bank_card_transaction_id value of bank card transaction ID
     */
    function setBankCardTransactionID( $vn_bank_card_transaction_id ) {
        $this->in_bank_card_transaction_id = $vn_bank_card_transaction_id;
    }
    
    /**
     * Return the value of bank account transaction ID
     * 
     * @return the value of bank account transaction ID
     */
    function getBankAccountTransactionID() {
        return $this->in_bank_account_transaction_id;
    }
    
    /**
     * Sets the value of bank account transaction ID
     * 
     * @param vn_bank_account_transaction_id value of bank account transaction ID
     */
    function setBankAccountTransactionID( $vn_bank_account_transaction_id ) {
        $this->in_bank_account_transaction_id = $vn_bank_account_transaction_id;
    }
    
    /**
     * Return the value of session ID
     * 
     * @return the value of session ID
     */
    function getSessionID() {
        return $this->is_session_id;
    }
    
    /**
     * Sets the value of session ID
     * 
     * @param vs_session_id value of session ID
     */
    function setSessionID( $vs_session_id ) {
        $this->is_session_id = $vs_session_id;
    }
    
    /**
     * Returns the JSON representation of this Customer Payment.
     * 
     * @return associated array : the json representation
     */
    function getJSON() {
        $o_json = array();
        
        if( !is_null( $this->is_customer_id ) ) {
            $o_json[ 'customer_payment_customer_id' ] = $this->is_customer_id;
        }
        
        if( !is_null( $this->is_source_did ) ) {
            $o_json[ 'customer_payment_source_did' ] = $this->is_source_did;
        }
        
        if( !is_null( $this->is_destination_did ) ) {
            $o_json[ 'customer_payment_destination_did' ] = $this->is_destination_did;
        }
        
        if( !is_null( $this->is_customer_account_number ) ) {
            $o_json[ 'customer_payment_customer_account_number' ] = $this->is_customer_account_number;
        }
        
        if( !is_null( $this->is_customer_zip_code ) ) {
            $o_json[ 'customer_payment_customer_zip_code' ] = $this->is_customer_zip_code;
        }
        
        if( !is_null( $this->id_payment_amount ) ) {
            $o_json[ 'customer_payment_payment_amount' ] = $this->id_payment_amount;
        }
        
        if( !is_null( $this->in_bank_card_transaction_id ) ) {
            $o_json[ 'bank_card_transaction_id' ] = $this->in_bank_card_transaction_id;
        }
        
        if( !is_null( $this->in_bank_account_transaction_id ) ) {
            $o_json[ 'bank_account_transaction_id' ] = $this->in_bank_account_transaction_id;
        }
        
        if( !is_null( $this->is_session_id ) ) {
            $o_json[ 'customer_payment_session_id' ] = $this->is_session_id;
        }
        
        return json_encode( $o_json );
    }
    
    /**
     * Constructs and returns an Customer Payment object from a given JSONObject.
     * 
     * @param o_json    json containing the CustomerPayment type
     * @return     the newly created CustomerPayment
     */
    static function buildFromJSON( $vo_json ) {
        $o_cp = new CustomerPayment();
        
        if( array_key_exists( 'customer_payment_customer_id', $vo_json ) ) {
            $o_cp->setCustomerID( $vo_json[ 'customer_payment_customer_id' ] );
        }
        
        if( array_key_exists( 'customer_payment_source_did', $vo_json ) ) {
            $o_cp->setSourceDID( $vo_json[ 'customer_payment_source_did' ] );
        }
        
        if( array_key_exists( 'customer_payment_destination_did', $vo_json ) ) {
            $o_cp->setDestinationDID( $vo_json[ 'customer_payment_destination_did' ] );
        }
        
        if( array_key_exists( 'customer_payment_customer_account_number', $vo_json ) ) {
            $o_cp->setCustomerAccountNumber( $vo_json[ 'customer_payment_customer_account_number' ] );
        }
        
        if( array_key_exists( 'customer_payment_customer_zip_code', $vo_json ) ) {
            $o_cp->setCustomerZipCode( $vo_json[ 'customer_payment_customer_zip_code' ] );
        }
        
        if( array_key_exists( 'customer_payment_payment_amount', $vo_json ) ) {
            $o_cp->setPaymentAmount( $vo_json[ 'customer_payment_payment_amount' ] );
        }
        
        if( array_key_exists( 'bank_card_transaction_id', $vo_json ) ) {
            $o_cp->setBankCardTransactionID( $vo_json[ 'bank_card_transaction_id' ] );
        }
        
        if( array_key_exists( 'bank_account_transaction_id', $vo_json ) ) {
            $o_cp->setBankAccountTransactionID( $vo_json[ 'bank_account_transaction_id' ] );
        }
        
        if( array_key_exists( 'customer_payment_session_id', $vo_json ) ) {
            $o_cp->setSessionID( $vo_json[ 'customer_payment_session_id' ] );
        }
        
        return $o_cp;
    }
}
