<?php

/**
 * Entity housing the attributes of a IVR information request
 */
class InformationRequest {
    
    private $is_source_did;
    private $is_destination_did;
    private $is_customer_account_number;
    private $is_customer_zip_code;
    
    /**
     * Default constructor
     */
    function __construct() {
        
    }
    
    /**
     * Return the value of source DID
     * 
     * @return the source did
     */
    function getSourceDID() {
        return $this->is_source_did;
    }
    
    /**
     * Sets the value of source DID
     * 
     * @param $vs_source_did value of source DID
     */
    function setSourceDID( $vs_source_did ) {
        $this->is_source_did = $vs_source_did;
    }
    
    /**
     * Return the value of destination DID
     * 
     * @return the destination DID
     */
    function getDestinationDID() {
        return $this->is_destination_did;
    } 
    
    /**
     * Sets the value of destination DID
     * 
     * @param $vs_destination_did value of destination DID
     */
    function setDestinationDID( $vs_destination_did ) {
        $this->is_destination_did = $vs_destination_did;
    }
    
    /**
     * Return the value of customer's account number
     * 
     * @return the customer account number
     */
    function getCustomerAccountNumber() {
        return $this->is_customer_account_number;
    }
    
    /**
     * Sets the value of customer's account number
     * 
     * @param $vs_customer_account_number value of customer account number
     */
    function setCustomerAccountNumber( $vs_customer_account_number ) {
        $this->is_customer_account_number = $vs_customer_account_number;
    }
    
    /**
     * Return the value of customer's zip code
     * 
     * @return the customer zip code
     */
    function getCustomerZipCode() {
        return $this->is_customer_zip_code;
    }
    
    /**
     * Sets the value of customer zip code
     * 
     * @param $vs_customer_zip_code value of customer zip code
     */
    function setCustomerZipCode( $vs_customer_zip_code ) {
        $this->is_customer_zip_code = $vs_customer_zip_code;
    }
    
    /**
     * Returns the JSON representation of this Information request.
     * 
     * @return associated array : the json representation
     */
    function getJSON() {
        $o_json = array();
        
        if( !is_null( $this->is_source_did ) ) {
            $o_json[ 'information_request_source_did' ] = $this->is_source_did;
        }
        
        if( !is_null( $this->is_destination_did ) ) {
            $o_json[ 'information_request_destination_did' ] = $this->is_destination_did;
        }
        
        if( !is_null( $this->is_customer_account_number ) ) {
            $o_json[ 'information_request_customer_account_number' ] = $this->is_customer_account_number;
        }
        
        if( !is_null( $this->is_customer_zip_code ) ) {
            $o_json[ 'information_request_customer_zip_code' ] = $this->is_customer_zip_code;
        }
        
        return json_encode( $o_json );
    }
    
    /**
     * Constructs and returns an InformationRequst  object from a given JSONObject.
     * 
     * @param o_json    json containing the InformationRequest type
     * @return     the newly created InformationRequest
     */
    static function buildFromJSON( $vo_json ) {
        $o_ir = new InformationRequest();
        
        if( array_key_exists( 'information_request_source_did', $vo_json ) ) {
            $o_ir->setSourceDID( $vo_json[ 'information_request_source_did' ] );
        }
        
        if( array_key_exists( 'information_request_destination_did', $vo_json ) ) {
            $o_ir->setDestinationDID( $vo_json[ 'information_request_destination_did' ] );
        }
        
        if( array_key_exists( 'information_request_customer_account_number', $vo_json ) ) {
            $o_ir->setCustomerAccountNumber( $vo_json[ 'information_request_customer_account_number' ] );
        }
        
        if( array_key_exists( 'information_request_customer_zip_code', $vo_json ) ) {
            $o_ir->setCustomerZipCode( $vo_json[ 'information_request_customer_zip_code' ] );
        }
        
        return $o_ir;
    }
}
