<?php
include_once 'Account.php';
include_once 'Location.php';

/**
 * Description of MerchantApplication
 * This class has the functions for the MerchantApplication
 */
class MerchantApplication {
    
    private $io_account;
    private $io_locations;
    private $io_response_messages;
    private $in_response_code;
    private $is_api_version;
    private $is_key;
    private $in_id;
    private $is_code;
    private $in_program_id;
    
    /*
     * $ib_auto_board is declared to check auto board status
     * $ib_auto_board_success is declared to check auto board success status
     * $is_sdk_username is the sdk user name to accept SDK User Name
     * $is_sdk_password is the sdk user password to accept SDK User Password
     * $is_sdk_key is the sdk user key to accept SDK User Key
     */    
    private $ib_auto_board;
    private $ib_auto_board_success;
    private $is_sdk_username;
    private $is_sdk_password;
    private $is_sdk_key;
    
    function __construct() {
        $this->io_locations = array();
        $this->io_response_messages = array();
    }
    
     
    /**
     * Returns true if MerchantApplication should be auto boarded, false otherwise
     */
    public function isAutoBoard() {
        return $this->ib_auto_board;
    }

    
    /**
     * Sets if the MerchantApplication should be auto boarded or not
     * @param vb_auto_board true if MerchantApplication should be auto boarded, false otherwise
     */
    public function setAutoBoard( $vb_auto_board ) {
        $this->ib_auto_board = $vb_auto_board;
    }
    
     /**
     * Returns true if the MerchantApplication's auto boarding was a success, false otherwise
     * @return true if MerchantApplication was auto boarded, false otherwise
     */
    public function isAutoBoardSuccess() {
        return $this->ib_auto_board_success;
    }

     /**
     * Sets if the MerchantApplication's auto boarding was a success or not
     * @param vb_auto_board_success true if MerchantApplication was auto boarded, false otherwise
     */
    public function setAutoBoardSuccess( $vb_auto_board_success ) {
        $this->ib_auto_board_success = $vb_auto_board_success;
    }

    /**
     * Returns the SDK Username of the Merchant. 
     * Only populated if the MerchantApplication successfully auto boarded
     * @return the SDK Username of the Merchant
     */
    public function getSDKUsername() {
        return $this->is_sdk_username;
    }

    /**
     * Sets the SDK Username of the Merchant. 
     * Only populated if the MerchantApplication successfully auto boarded
     * @param vs_sdk_username the SDK Username of the Merchant
     */
    public function setSDKUsername( $vs_sdk_username ) {
        $this->is_sdk_username = $vs_sdk_username;
    }

    /**
     * Returns the SDK Password of the Merchant. 
     * Only populated if the MerchantApplication successfully auto boarded
     * @return the SDK Password of the Merchant
     */
    public function getSDKPassword() {
        return $this->is_sdk_password;
    }

    /**
     * Sets the SDK Password of the Merchant. 
     * Only populated if the MerchantApplication successfully auto boarded
     * @param vs_sdk_password the sdk password
     */
    public function setSDKPassword( $vs_sdk_password ) {
        $this->is_sdk_password = $vs_sdk_password;
    }
    
    /**
     * Returns the SDK Key of the Merchant. 
     * Only populated if the MerchantApplication successfully auto boarded
     * @return the SDK Key of the Merchant
     */
    public function getSDKKey() {
        return $this->is_sdk_key;
    }

    /**
     * Sets the SDK Key of the Merchant. 
     * Only populated if the MerchantApplication successfully auto boarded
     * @param vs_sdk_key the sdk key
     */
    public function setSDKKey( $vs_sdk_key ) {
        $this->is_sdk_key = $vs_sdk_key;
    }
       
    public function getAccount() {
        return $this->io_account;
    }

    public function setAccount( $vo_account ) {
        $this->io_account = $vo_account;
    }

    public function getLocations() {
        return $this->io_locations;
    }

    public function addLocation( $vo_location ) {
        array_push( $this->io_locations, $vo_location );
    }
    
    public function getResponseMessages() {
        return $this->io_response_messages;
    }
    
    public function addResponseMessage( $vs_field, $vs_response_message ) {
        $this->io_response_messages[$vs_field] = $vs_response_message;
    }

    public function getResponseCode() {
        return $this->in_response_code;
    }
    
    public function setResponseCode( $vn_response_code ) {
        $this->in_response_code = $vn_response_code;
    }
    
    public function getApiVersion() {
        return $this->is_api_version;
    }

    public function setApiVersion( $vs_api_version ) {
        $this->is_api_version = $vs_api_version;
    }
    
    public function getKey() {
        return $this->is_key;
    }
    
    public function setKey( $vs_key ) {
        $this->is_key = $vs_key;
    }

    public function getId() {
        return $this->in_id;
    }
    
    public function setId( $vn_id ) {
        $this->in_id = $vn_id;
    }

    public function getCode() {
        return $this->is_code;
    }
    
    public function setCode( $vs_code ) {
        $this->is_code = $vs_code;
    }
    
    /**
     * Sets the value of program ID
     * 
     * @param $vn_program_id new value of program ID
     */
    public function setProgramID( $vn_program_id ) {
        $this->in_program_id = $vn_program_id;
    }

    /**
     * Returns the value of program ID
     * 
     * @return value of program ID
     */
    public function getProgramID() {
        return $this->in_program_id;
    }
    
    /**
     * Returns a JSON representation of the MerchantApplication
     * 
     * @return JSON object of the MerchantApplication
     */
    public function getJSON() {
        
        $o_json = array();
        
        if( !is_null( $this->io_account ) ) { 
            $o_json[ "merchant_application_account" ] = $this->io_account->getJSON(); 
        }

        $o_locations = array();
        for( $n_index = 0, $n_size = count( $this->io_locations ); $n_index < $n_size; $n_index++ ) {
            $o_location = $this->io_locations[$n_index];
            array_push( $o_locations, $o_location->getJSON() );
        }
        $o_json[ "merchant_application_locations"] = $o_locations;
        
        if( count( $this->io_response_messages) > 0) {
            $o_json[ "merchant_application_response_messages"] = $this->io_response_messages;
        }
        
        if( !is_null( $this->in_response_code ) ) { 
            $o_json[ "merchant_application_response_code" ] = $this->in_response_code; 
        }
        
        if( !is_null( $this->is_api_version ) ) { 
            $o_json[ "merchant_application_api_version" ] = $this->is_api_version; 
        }
        if( !is_null( $this->is_key ) ) { 
            $o_json[ "merchant_application_key" ] = $this->is_key; 
        }
        if( !is_null( $this->in_id ) ) { 
            $o_json[ "merchant_application_id" ] = $this->in_id; 
        }
        if( !is_null( $this->is_code ) ) { 
            $o_json[ "merchant_application_code" ] = $this->is_code;             
        }
        
        if( !is_null( $this->ib_auto_board ) ) { 
            $o_json[ "merchant_application_auto_board" ] = $this->ib_auto_board; 
        }
        if( !is_null( $this->ib_auto_board_success ) ) { 
            $o_json[ "merchant_application_auto_board_success" ] = $this->ib_auto_board_success; 
        }
        if( !is_null( $this->is_sdk_username ) ) { 
            $o_json[ "merchant_application_sdk_username" ] = $this->is_sdk_username; 
        }
        if( !is_null( $this->is_sdk_password ) ) { 
            $o_json[ "merchant_application_sdk_password" ] = $this->is_sdk_password;
        }
        if( !is_null( $this->is_sdk_key ) ) { 
            $o_json[ "merchant_application_sdk_key" ] = $this->is_sdk_key; 
        }
        if( !is_null( $this->in_program_id) ) {
            $o_json[ "merchant_application_program_id" ] = $this->in_program_id;
        }
        
        return json_encode( $o_json );
    }
    
    
    /**
     * Builds a MerchantApplication object from the given JSON object
     * @param vo_json the JSON representation of the MerchantApplication
     * @return the MerchantApplication
     */    
    public static function buildFromJSON( $vo_json ) {
        
        $o_merch_app = new MerchantApplication();
        
        if( array_key_exists( "merchant_application_account", $vo_json ) ) {
//        if ( array_key_exists( "merchant_application_account", $vo_json ) ) { 
//        if
//            var_dump( $vo_json[ "merchant_application_account" ] );
            $o_merch_app->setAccount( Account::buildFromJSON( $vo_json[ "merchant_application_account" ] ) );
        } else {
            //var_dump( "welp, this sucks" );
            //var_dump( $vo_json["merchant_application_accounts"] );
        }
        if( array_key_exists( "merchant_application_locations", $vo_json ) ) {
            $o_locations = $vo_json[ "merchant_application_locations" ];
            for ( $n_index = 0, $n_size = count( $o_locations ); $n_index < $n_size; $n_index++ ) {
                //var_dump( $o_locations[ $n_index ] );
                $o_location = Location::buildFromJSON( $o_locations[ $n_index ] );
                $o_merch_app->addLocation( $o_location );
            }
        }
        
        if( array_key_exists( "merchant_application_response_messages", $vo_json ) ) {
            foreach($vo_json[ "merchant_application_response_messages" ] as $key => $val) {
                $o_merch_app->addResponseMessage($key, $val);
            
            }
//            $o_response_messages = $vo_json[ "merchant_application_response_messages" ];
//            for ( $n_index = 0, $n_size = count( $o_response_messages ); $n_index < $n_size; $n_index++ ) {
//                $o_merch_app->addResponseMessage( $o_response_messages[ $n_index ] );
//            }
        }
        
        if( array_key_exists( "merchant_application_response_code", $vo_json ) ) { 
            $o_merch_app->setResponseCode( $vo_json[ "merchant_application_response_code" ] ); 
        }
        if( array_key_exists( "merchant_application_api_version", $vo_json ) ) { 
            $o_merch_app->setApiVersion( $vo_json[ "merchant_application_api_version" ] ); 
        }
        if( array_key_exists( "merchant_application_key", $vo_json ) ) { 
            $o_merch_app->setKey( $vo_json[ "merchant_application_key" ] ); 
        }
        if( array_key_exists( "merchant_application_id", $vo_json ) ) { 
            $o_merch_app->setId( $vo_json[ "merchant_application_id" ] );             
        }
        if( array_key_exists( "merchant_application_code", $vo_json ) ) { 
            $o_merch_app->setCode( $vo_json[ "merchant_application_code" ] ); 
        }
        
        if( array_key_exists( "merchant_application_auto_board", $vo_json ) ) { 
            $o_merch_app->setAutoBoard( $vo_json[ "merchant_application_auto_board" ] );             
        }
        if( array_key_exists( "merchant_application_auto_board_success", $vo_json ) ) { 
            $o_merch_app->setAutoBoardSuccess( $vo_json[ "merchant_application_auto_board_success" ] ); 
        }
        if( array_key_exists( "merchant_application_sdk_username", $vo_json ) ) { 
            $o_merch_app->setSDKUsername( $vo_json[ "merchant_application_sdk_username" ] ); 
        }
        if( array_key_exists( "merchant_application_sdk_password", $vo_json ) ) { 
            $o_merch_app->setSDKPassword( $vo_json[ "merchant_application_sdk_password" ] ); 
        }
        if( array_key_exists( "merchant_application_sdk_key", $vo_json ) ) { 
            $o_merch_app->setSDKKey( $vo_json[ "merchant_application_sdk_key" ] ); 
        }
        if( array_key_exists( "merchant_application_program_id", $vo_json ) ) {
            $o_merch_app->setProgramID( $vo_json[ "merchant_application_program_id" ] );
        }
        
        return $o_merch_app;
    }
    
}

?>
