<?php


/**
 * Description of PushNotification
 *
 */
class PushNotification {
    
    static $XS_PN_ACH_CHANGE = "ACH CHANGE";
    static $XS_PN_TYPE_BANK_CARD_TRANSACTION = "BANK CARD TRANSACTION";
    static $XS_PN_SETTLEMENT_BATCH_CHANGE = "SETTLEMENT BATCH CHANGE";
    static $XS_PN_TYPE_BANK_CARD_UPDATE = "BANK_CARD_UPDATE";
    static $XS_PN_TYPE_PING = "PING";
    static $XS_PN_TYPE_JSON = "JSON";
    static $XS_PN_TYPE_INCOMING_CHARGEBACK = "INCOMING CHARGEBACK";
    static $XS_PN_TYPE_MERCHANT_CREDENTIALS = "MERCHANT CREDENTIALS";
    static $XS_PN_TYPE_INFORMATION_REQUEST = "INFORMATION REQUEST";
    static $XS_PN_TYPE_CUSTOMER_PAYMENT = "CUSTOMER PAYMENT";
    
    protected $is_type;
    private $io_settlement_batch;
    private $io_bank_account_transaction;
    private $io_bank_card_transaction;
    private $in_id;
    private $io_bank_card_update;
    private $io_incoming_chargeback;
    private $io_merchant_application;
    private $io_information_request;
    private $io_customer_payment;
    
    public function getBankAccountTransaction() {
        return $this->io_bank_account_transaction;
    }
    
    public function setBankAccountTransaction( BankAccountTransaction $vo_bat ) {
        $this->io_bank_account_transaction = $vo_bat;
        $this->is_type = PushNotification::$XS_PN_ACH_CHANGE;
    }
    
    /**
     * Returns the BankCardTransaction on the PushNotification
     * 
     * @return BankCardTransaction
     */
    public function getBankCardTransaction() {
        return $this->io_bank_card_transaction;
    }
    
    /**
     * Sets the BankCardTransaction on the PushNotification and sets the type to $XS_PN_TYPE_BANK_CARD_TRANSACTION.
     * 
     * @param BankCardTransaction $vo_bct
     */
    public function setBankCardTransaction( BankCardTransaction $vo_bct ) {
        $this->io_bank_card_transaction = $vo_bct;
        $this->is_type = PushNotification::$XS_PN_TYPE_BANK_CARD_TRANSACTION;
    }
    
    public function getSettlementBatch() {
        return $this->io_settlement_batch;
    }
    
    public function setSettlementBatch( SettlementBatch $vo_settlement_batch ) {
        $this->io_settlement_batch = $vo_settlement_batch;
        $this->is_type = PushNotification::$XS_PN_SETTLEMENT_BATCH_CHANGE;
    }
    
    public function getNotificationType() {
        return $this->is_type;
    }
    
    public function isNotificationType( $vs_type ) {
        return $this->is_type === $vs_type;
    }

    /**
     * Sets the type to ping
     */
    public function setPING() {
       $this->is_type = self::$XS_PN_TYPE_PING;
    }
    
    /**
     * Returns the PushNotification ID
     * 
     * @return the PushNotification ID
     */
    public function getID() {
        return $this->in_id;
    }
    
    /**
     * Sets the PushNotification ID
     * 
     * @param vn_id the PushNotification ID
     */
    public function setID( $vn_id ) {
        $this->in_id = $vn_id;
    }
    
    public function getBankCardUpdate() {
        return $this->io_bank_card_update;
    }
    
    /**
     * sets the BankCard Update 
     * @param type $vo_bank_card_update the BankCard Update
     */
    public function setBankCardUpdate(BankCardUpdate $vo_bank_card_update ) {
        $this->io_bank_card_update = $vo_bank_card_update;
        $this->is_type = self::$XS_PN_TYPE_BANK_CARD_UPDATE;
    }
    
    /**
     * returns the incoming chargeback
     * @return the incoming chargeback
     */
    public function getIncomingChargeback() {
        return $this->io_incoming_chargeback;
    }
    
    /**
     * sets the incoming chargeback
     * @param type $vo_incoming_chargeback the incoming chargeback
     */
    public function setIncomingChargeback(Chargeback $vo_incoming_chargeback ) {
        $this->io_incoming_chargeback = $vo_incoming_chargeback;
        $this->is_type = PushNotification::$XS_PN_TYPE_INCOMING_CHARGEBACK;
    }
    
    /**
     * returns the merchant application
     * @return the merchant application
     */
    public function getMerchantApplication() {
        return $this->io_merchant_application;
    }
    
    /**
     * sets the merchant application
     * @param type vo_merchant_application The merchant application
     */
    public function setMerchantApplication( MerchantApplication $vo_merchant_application) {
        $this->io_merchant_application = $vo_merchant_application;
        $this->is_type = PushNotification::$XS_PN_TYPE_MERCHANT_CREDENTIALS;
    }
    
    /**
     * Returns the Information Request object
     * 
     * @return the value of Information Request
     */
    public function getInformationRequest() {
        return $this->io_information_request;
    }
    
    /**
     * Sets the value of Information Request
     * 
     * @param InformationRequest $vo_information_request the new value of Information Request
     */
    public function setInformationRequest( InformationRequest $vo_information_request ) {
        $this->io_information_request = $vo_information_request;
        $this->is_type = PushNotification::$XS_PN_TYPE_INFORMATION_REQUEST;
    }
    
    /**
     * Returns the Customer Payment object
     * 
     * @return the value of Customer Payment
     */
    public function getCustomerPayment() {
        return $this->io_customer_payment;
    }
    
    /**
     * Sets the value of Customer Payment
     * 
     * @param CustomerPayment $vo_customer_payment the new value of Customer Payment
     */
    public function setCustomerPayment( CustomerPayment $vo_customer_payment ) {
        $this->io_customer_payment = $vo_customer_payment;
        $this->is_type = PushNotification::$XS_PN_TYPE_CUSTOMER_PAYMENT;
    }

    /**
     * Returns a json array of the push notification data
     * 
     * @return json array with push notification data
     */
    public function getJSON() {
        
        $o_return = array();
        
        if( !is_null( $this->is_type ) ) { 
            $o_return[ "push_notification_type" ] = json_decode( $this->is_type, true ); 
        }
        
        if( !is_null( $this->io_settlement_batch ) ) { 
            $o_return[ "push_notification_settlement_batch" ] = json_decode( $this->io_settlement_batch.getJSON(), true ); 
        }
        
        if( !is_null( $this->io_bank_account_transaction ) ) { 
            $o_return[ "push_notification_bank_account_transaction" ] = json_decode( $this->io_bank_account_transaction->getJSON(), true ); 
        }
        
        if( !is_null( $this->io_bank_card_transaction ) ) {  
            $o_return[ "push_notification_bank_card_transaction" ] = json_decode( $this->io_bank_card_transaction->getJSON(), true );       
        }
        
        if( !is_null( $this->in_id ) ) {            
            $o_return[ "push_notification_id" ] = json_decode( $this->in_id, true );            
        }
        
        if( !is_null( $this->io_bank_card_update ) ) {             
            $o_return[ "push_notification_bank_card_update" ] = json_decode( $this->io_bank_card_update->getJSON(), true );            
        }
        
        if( !is_null( $this->io_incoming_chargeback ) ) {           
            $o_return[ "push_notification_chargeback" ] = json_decode( $this->io_incoming_chargeback->getJSON(), true );          
        }
        
        if( !is_null( $this->io_merchant_application) ) {          
            $o_return[ "merchant_application" ] = json_decode( $this->io_merchant_application->getJSON(), true );         
        }
        
        if( !is_null( $this->io_information_request ) ) {
            $o_return[ "information_request" ] = json_decode( $this->io_information_request->getJSON(), true );
        }
        
        if( !is_null( $this->io_customer_payment ) ) {
            $o_return[ "customer_payment" ] = json_decode( $this->io_customer_payment->getJSON(), true );
        }
        
        return json_encode( $o_return );
    }
    
    /**
     * Builds a PushNotification object from the given JSON object
     * 
     * @param type $vo_json the JSON representation of the PushNotification
     * @return PushNotification a push notification object
     */
    public static function buildFromJSON( $vo_json ) {
        
        $o_return = new PushNotification();
        
        if( array_key_exists( "push_notification_type", $vo_json ) && $vo_json[ "push_notification_type" ] == "PING" ) { 
            $o_return->setPING(); 
        }
        
        if( array_key_exists( "push_notification_type", $vo_json ) && $vo_json[ "push_notification_type" ] == "INCOMING CHARGEBACK" ) {
            if( array_key_exists( "push_notification_chargeback", $vo_json ) ) {
                $o_return->setIncomingChargeback( Chargeback::buildFromJSON( $vo_json[ "push_notification_chargeback" ] ) );
            }
        }
        
        if( array_key_exists( "push_notification_settlement_batch", $vo_json ) ) { 
            $o_return->setSettlementBatch( SettlementBatch::buildFromJSON( $vo_json[ "push_notification_settlement_batch" ] ) );             
        }
        
        if( array_key_exists( "push_notification_bank_account_transaction", $vo_json ) ) { 
            $o_return->setBankAccountTransaction( BankAccountTransaction::buildFromJSON( $vo_json[ "push_notification_bank_account_transaction" ] ) );             
        }
        
        if( array_key_exists( "push_notification_bank_card_transaction", $vo_json ) ) { 
            $o_return->setBankCardTransaction( BankCardTransaction::buildFromJSON( $vo_json[ "push_notification_bank_card_transaction" ] ) );          
        }
        
        if( array_key_exists( "push_notification_id", $vo_json ) ) { 
            $o_return->setID( $vo_json[ "push_notification_id" ] );        
        }
        
        if( array_key_exists( "push_notification_bank_card_update", $vo_json ) ) { 
            $o_return->setBankCardUpdate( BankCardUpdate::buildFromJSON( $vo_json[ "push_notification_bank_card_update" ] ) ); 
        }
        
        if( array_key_exists( "merchant_application", $vo_json ) ) { 
            $o_return->setMerchantApplication( MerchantApplication::buildFromJSON( $vo_json[ "merchant_application" ] ) );             
        }
        
        if( array_key_exists( "information_request", $vo_json ) ) {
            $o_return->setInformationRequest( InformationRequest::buildFromJSON( $vo_json[ "information_request" ] ) );
        }
        
        if( array_key_exists( "customer_payment", $vo_json ) ) {
            $o_return->setCustomerPayment( CustomerPayment::buildFromJSON( $vo_json[ "customer_payment" ] ) );
        }
        
        return $o_return;
    }
}
